#include <cstdio>
#include <cmath>
#include <climits>
#include <cstdlib>
#include <ctime>

const int TESTS = 10000;
const double EPS = 1e-13;

inline double sqr(double _a)
{
 return _a * _a;
}

inline double eq(double a, double b)
{
 return fabs(a - b) < EPS;
}

struct airport  // structure for airports
{
    double x,y,z;
 airport(){}
 airport(double _x, double _y, double _z): x(_x), y(_y), z(_z) {}
};

// points can be treated as radius vectors for convenience

airport operator+ (const airport &a, const airport &b)
{
 return airport(a.x + b.x, a.y + b.y, a.z + b.z);
}

airport operator* (const airport &a, const double &c)
{
 return airport(a.x * c, a.y * c, a.z * c);
}

void print(const airport &a)
{
 printf("%lf %lf %lf\n", a.x, a.y, a.z);
}

double dist2(const airport &a, const airport &b)
{
 return sqr(a.x - b.x) + sqr(a.y - b.y) + sqr(a.z - b.z);
}

double dist(const airport &a, const airport &b)
{
 return sqrt(sqr(a.x - b.x) + sqr(a.y - b.y) + sqr(a.z - b.z));
}

double len(const airport &a)
{
 return dist(airport(0, 0, 0), a);
}

double RR,R;  // squared radius, radius

double arclength_cos(const airport &A,const airport &B)  // calculate distance of flight using law of cosines
{
    double AB = sqr(A.x-B.x) + sqr(A.y-B.y) + sqr(A.z-B.z);
    double cosa = 1. - AB / (2.*RR);
    return R*acos(cosa);
}

double arclength_sin(const airport &a, const airport &b)  // calculate distance of flight using law of sines
{
 double d = dist(a, b);
 double sina2 = d / (2 * R);
// printf("sin alfa/2: %lf\n", sina2);
 return R * asin(sina2) * 2;
}

double arclength_3d_angle(const airport &a, const airport &b)	// calculate distance of flight using dot product ( = product of vector lengths times the cosine of their angle )
{
 double dot = a.x * b.x + a.y * b.y + a.z * b.z;
 double cosa = dot / (len(a) * len(b));
 return acos(cosa) * R;
}

double arclength_binary(const airport &a, const airport &b)   // using binary search
{
 double mply = 1;
 airport tmp = b;
 int iter = 19, i = 0;
 while(1)
 {
  double d = dist(a, tmp);
//  if(d < 1e-13) return d * mply;
  i ++;
  if(i == iter)
   return d * pow(2., (double)iter - 1);

  airport mid = (a + tmp) * .5;
  double coef = R / sqrt(sqr(mid.x) + sqr(mid.y) + sqr(mid.z));

  mid = mid * coef;
  tmp = mid;
  mply *= 2.;
 }
 return -1;
}

double arclength_binary_squared(const airport &a, const airport &b)  // using two binary searches
{
 double mply = 1;
 airport tmp = b;
 int iter = 19, i = 0;
 while(1)
 {
  double d = dist(a, tmp);
//  if(d < 1e-13) return d * mply;
  i ++;
  if(i == iter)
   return d * pow(2., (double)iter - 1);

  airport mid = (a + tmp) * .5;
  double l = 0, r = 1e10;
  for(int i = 0; i < 20; i ++)
  {
   double m = (l + r) * .5;
   airport cur = mid * m;

   if(len(cur) > R)
    r = m;
   else
    l = m;
  }


  mid = mid * l;
  tmp = mid;
  mply *= 2.;
 }
 return -1;
}

airport gen()				// generate a number of test cases to compare efficeincy of all methods
{
 double z = (double) rand() / RAND_MAX;
 z = z * 2. - 1.;
 double r = sqrt(1. - z * z);
 double t = (double) rand() / RAND_MAX;
 t = t * 2. * M_PI;
 double x = r * cos(t);
 double y = r * sin(t);
 x *= R; y *= R; z *= R;
// printf("%.18lf %.18lf %.18lf %d\n",x,y,z,refuel[i+1]);
 return airport(x, y, z);
}

int main()
{
 R = 1;
 RR = R * R;
 volatile double d;
 for(int t = 0; t < TESTS; t ++)
 {
  airport a = gen(), b = gen();

  d = arclength_sin(a, b);
/*  double d1 = arclength_3d_angle(a, b);
  double d2 = arclength_sin(a, b);
  if(!eq(d1, d2))
  {
   print(a);
   print(b);
   printf("%.10lf\n%.10lf\n", d1, d2);
   printf("%e", fabs(d1 - d2));
   getc(stdin);
  }
  else
   printf("test %d passed\n", t); */
 }
 fprintf(stderr, "%lf\n", clock() / double(CLOCKS_PER_SEC));
 return 0;
}
